/*
    ChibiOS/RT - Copyright (C) 2006,2007,2008,2009,2010,
                 2011,2012,2013 Giovanni Di Sirio.

    This file is part of ChibiOS/RT.

    ChibiOS/RT is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    ChibiOS/RT is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "ch.h"
#include "hal.h"
#include "test.h"
#include "lcd3310.h"

/*
 * SPI configuration (1MHz, CPHA=0, CPOL=0).
 */
static SPIConfig spicfg = {
  NULL,
  GPIO2,
  GPIO2_LCD_SS,
  CR0_DSS8BIT | CR0_FRFSPI | CR0_CLOCKRATE(0),
  36
};

/*
 * LEDs blinker thread, times are in milliseconds.
 */
static WORKING_AREA(waThread1, 128);
static msg_t Thread1(void *arg) {

  (void)arg;
  chRegSetThreadName("blinker1");
  while (TRUE) {
    palClearPad(GPIO1, GPIO1_LED1);
    palSetPad(GPIO1, GPIO1_LED2);
    chThdSleepMilliseconds(500);
    palSetPad(GPIO1, GPIO1_LED1);
    palClearPad(GPIO1, GPIO1_LED2);
    chThdSleepMilliseconds(500);
  }

  return 0;
}


/*
 * Application entry point.
 */
int main(void) {
  uint8_t i;

  /*
   * System initializations.
   * - HAL initialization, this also initializes the configured device drivers
   *   and performs the board-specific initializations.
   * - Kernel initialization, the main() function becomes a thread and the
   *   RTOS is active.
   */
  halInit();
  chSysInit();

  /*
   * Activates the SD1 and SPI1 drivers.
   */
  sdStart(&SD1, NULL);                  /* Default: 38400,8,N,1.            */
  spiStart(&SPID1, &spicfg);
  lcd3310Init(&SPID1);
  lcd3310Contrast(&SPID1, 0x70);

  /*
   * Creates the blinker threads.
   */
  chThdCreateStatic(waThread1, sizeof(waThread1), NORMALPRIO, Thread1, NULL);

  /*
   * Normal main() thread activity, in this demo it updates Nokia
   * 3310 LCD.
   */
  i = 0;
  while (TRUE) {
    if (!palReadPad(GPIO2, GPIO2_SW_USER1))
       TestThread(&SD1);
    lcd3310SetPosXY(&SPID1, 0, 2);
    lcd3310RotateText(&SPID1, (unsigned char *)"     *** ChibiOS ***    ", i);
    chThdSleepMilliseconds(500);
    (i > 22) ? i = 0 : i++;
  }
}
